/* $Id: dummy.c,v 1.4 1998/12/01 12:50:35 kas Exp $ */

/*
 *  Copyright (C) 1995-1997  Jan "Yenya" Kasprzak <kas@fi.muni.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/module.h>
#include <linux/config.h>

#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include "syncppp.h"

static int sdummy_open (struct device *d)
{
	int err;

	printk(KERN_INFO "%s: open\n", d->name);

	err = sppp_open(d);
	if (err)
		return err;
	d->tbusy = 0;
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
	return 0;
}

static int sdummy_close (struct device *d)
{
	printk(KERN_INFO "%s: close\n", d->name);

	sppp_close(d);
	d->tbusy = 1;
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
	return 0;
}

static int sdummy_ioctl (struct device *d, struct ifreq *ifr, int cmd)
{
	printk(KERN_INFO "%s: ioctl 0x%x\n", d->name, cmd);
	return sppp_do_ioctl(d, ifr, cmd);
}

static int sdummy_xmit (struct sk_buff *skb, struct device *d)
{
	int i;

	printk(KERN_INFO "%s: xmit %d bytes", d->name, skb->len);
	for (i=0; i<skb->len; i++)
		printk(" %02x", skb->data[i]&0xff);
	printk("\n");
	dev_kfree_skb(skb);
	return 0;
}

static struct ppp_device d;
static int sdummy_init(struct device *dev)
{
	printk(KERN_INFO "%s: sdummy_init\n", dev->name);
	dev->open = sdummy_open;
	dev->stop = sdummy_close;
	dev->hard_start_xmit = sdummy_xmit;
	dev->do_ioctl = sdummy_ioctl;
	dev->priv = &d.sppp;
	d.sppp.pp_flags = PP_CISCO;
	dev->open = sdummy_open;
	dev->stop = sdummy_close;
	dev->hard_start_xmit = sdummy_xmit;
	dev->do_ioctl = sdummy_ioctl;
	dev_init_buffers(dev);
	sppp_attach(&d);
	return 0;
}

int init_module(void)
{
	memset(&d.dev,0,sizeof(d));
	d.dev.name = "sdummy0";
	d.dev.init = sdummy_init;
	printk(KERN_INFO "%s: init_module\n", d.dev.name);
	return register_netdev(&d.dev);
}

void cleanup_module(void)
{
	printk(KERN_INFO "%s: cleanup_module\n", d.dev.name);
	sppp_detach(&d.dev);
	unregister_netdev(&d.dev);
}
