/* $Id: srctl.c,v 1.5 1999/01/06 14:02:49 kas Exp $ */

/*
 *  Copyright (C) 1995-1997  Jan "Yenya" Kasprzak <kas@fi.muni.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <fcntl.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include "syncppp.h"

#include "cosa.h"

static char *devname;

int open_fd(dev)
char *dev;
{
	int fd;

	devname = dev;

	if (strchr(dev, '/')) {	/* Character device */
		fd = open(dev, O_RDONLY);
	} else {	/* Network interface */
		fd = socket(AF_INET, SOCK_DGRAM, 0);
	}
	return fd;
}

int do_ioctl(fd, cmd, arg)
int fd;
int cmd;
unsigned long arg;
{
	char *dev = devname;
	if (strchr(dev, '/')) { /* Character device ioctl */
		return ioctl(fd, cmd, arg);
	} else {
		struct ifreq ifr;
		strncpy(ifr.ifr_name, dev, IFNAMSIZ);
		ifr.ifr_ifru.ifru_data = (void *)arg;
		return ioctl(fd, cmd, &ifr);
	}
}

void usage(name)
char *name;
{
	fprintf(stderr, "Usage: %s <device> <command> [ <command_parameters> ]\n",
		name);
	fprintf(stderr, "The following commands are recognized:\n"
		"\treset\n"
		"\tdownload <addr> <file>\n"
		"\thexload <file>\n"
	/*	"\treadmem <addr> <length> <file>\n"  not yet */
		"\tgetidstring\n"
		"\tgettype\n"
		"\tstart <addr>\n"
#if 0
		"\tmodule_dec\n"
		"\tmodule_inc\n"
#endif
		"\tnr_cards\n"
		"\tnr_channels\n"
		"\tsyncppp\n"
		"\thdlc\n"
		"\tdebug\n"
		"\tnodebug\n"
		"\tbmget\n"
		"\tbmset [0|1]\n"
		"The <device> can be the character device (its name must contain a slash),\n"
		"or the network interface name (does not contain a slash).\n");
	exit(1);
}

int download(int fd, struct cosa_download *d)
{
#ifdef VERIFY_DOWNLOAD
	char *firmware;
	int s, i;
#endif
	if (do_ioctl(fd, COSAIODOWNLD, d) == -1) {
		perror("COSAIODOWNLD");
		return -1;
	}
#ifdef VERIFY_DOWNLOAD
	firmware = d->code;
	if ((d->code = malloc(d->len)) == (char *)0) {
		fputs("Cannot allocate memory for verifying", stderr);
		return -1;
	}
	if (do_ioctl(fd, COSAIORMEM, d) == -1) {
		perror("COSAIORMEM");
		return -1;
	}
	s=0;
	for(i=0; i<d->len; i++)
		if (firmware[i] != d->code[i]) {
			fprintf(stderr, "Verifying download: byte 0x%04x different (0x%02x vs 0x%02x)\n",
				i, firmware[i]&0xff, d->code[i]&0xff);
			s=-1;
		}
	return s;
#else
	return 0;
#endif
}
	
int main(argc, argv)
int argc;
char **argv;
{
	int fd;

	if (argc < 3) usage(*argv);
	
	if ((fd = open_fd(argv[1])) == -1) {
		perror("Opening device");
		return -1;
	}

	if (!strcmp(argv[2], "reset")) {
		if (do_ioctl(fd, COSAIORSET, 0) == -1) {
			perror("resetting cosa");
			return -1;
		}
		return 0;
	} else if (!strcmp(argv[2], "download") && argc == 5) {
		int fd1, b, s, i;
		struct cosa_download d;
		struct stat st;
		i=sscanf(argv[3], "%i", &d.addr);

		if ((fd1 = open(argv[4], O_RDONLY)) == -1) {
			perror ("opening image");
			return -1;
		}
		if (fstat(fd1, &st) != 0) {
			perror ("fstat");
			return -1;
		}
		if ((d.code = malloc(st.st_size)) == (char *)0) {
			fputs("Cannot allocate memory", stderr);
			return -1;
		}

		for (b=0,s=st.st_size; s; s-=i, b+=i)
			if ((i=read(fd1, d.code+b, s)) <= 0) {
				perror("reading code");
				return -1;
			}
		close(fd1);

		d.len = st.st_size;
		return download(fd, &d);
	} else if (!strcmp(argv[2], "start") && argc == 4) {
		int addr;
		sscanf(argv[3], "%i", &addr);
		if (do_ioctl(fd, COSAIOSTRT, addr) == -1) {
			perror("starting microcode");
			return -1;
		}
		return 0;
	} else if (!strcmp(argv[2], "hexload") && argc == 4) {
		struct cosa_download d;
		FILE *fp;
		int lnum = 0;
		int eol;
		if (!(fp = fopen(argv[3], "r"))) {
			perror("opening data file");
			return -1;
		}
		while (fgetc(fp) == ':') {
			int len, addr, sum=0;
			int hi,lo;
			lnum++;
			if (fscanf(fp,"%02x", &len) != 1) goto badformat;
			if (!len) break;
			sum+=len&0xff;
			if (fscanf(fp, "%02x%02x", &hi, &lo) != 2)
				goto badformat;
			addr = lo+(hi<<8);
			sum+=(lo+hi)&0xff;
			if (lnum == 1) {
				d.addr = addr;
				d.len = 0;
				d.code = malloc(len);
			} else {
				if (d.addr+d.len != addr)
					goto badformat;
				d.code = realloc(d.code, d.len+len);
			}
			if (fscanf(fp, "%02x", &hi) != 1) goto badformat;
			sum += hi&0xff;
			while(len--) {
				int c;
				if (fscanf(fp, "%02x", &c) != 1) goto badformat;
				d.code[d.len++] = c&0xff;
				sum += c&0xff;
			}
			if (fscanf(fp, "%02x", &hi) != 1) goto badformat;
			sum += hi&0xff;
			if (hi != 0 && (sum&0xff)!=0)
				printf("Bad checksum 0x%02x at line %d\n",
					sum&0xff, lnum);
			/* We need to handle DOS end-of-lines correctly */
			eol = fgetc(fp);
			if (eol != '\n') {
				if (eol != '\r') goto badformat;
				else if (fgetc(fp) != '\n') goto badformat;
			}
		}
#if 0
		for (i=0; i<d.len; i++) 
			printf("%02x", d.code[i]&0xff);
#endif
		return download(fd, &d);
badformat:
		fprintf(stderr, "Invalid file format at line %d\n", lnum);
		return -1;
	} else if (!strcmp(argv[2], "gettype") && argc == 3) {
		char type[COSA_MAX_ID_STRING+1];
		if (do_ioctl(fd, COSAIORTYPE, type) == -1) {
			perror ("COSAIORTYPE");
			return -1;
		}
		type[COSA_MAX_ID_STRING] = '\0';
		puts(type);
		return 0;
	} else if (!strcmp(argv[2], "getidstring") && argc == 3) {
		char type[COSA_MAX_ID_STRING+1];
		if (do_ioctl(fd, COSAIORIDSTR, type) == -1) {
			perror ("COSAIORIDSTR");
			return -1;
		}
		type[COSA_MAX_ID_STRING] = '\0';
		puts(type);
		return 0;
#if 0
	} else if (!strcmp(argv[2], "module_inc") && argc == 3) {
		if (do_ioctl(fd, COSAIOMINC, 0) == -1) {
			perror ("COSAIOMINC");
			return -1;
		}
		return 0;
	} else if (!strcmp(argv[2], "module_dec") && argc == 3) {
		if (do_ioctl(fd, COSAIOMDEC, 0) == -1) {
			perror ("COSAIOMDEC");
			return -1;
		}
		return 0;
#endif
	} else if (!strcmp(argv[2], "nr_cards") && argc == 3) {
		int i;
		if ((i=do_ioctl(fd, COSAIONRCARDS, 0)) < 0) {
			perror ("COSAIONRCARDS");
			return -1;
		} else
			printf("%d\n", i);
		return 0;
	} else if (!strcmp(argv[2], "nr_channels") && argc == 3) {
		int i;
		if ((i=do_ioctl(fd, COSAIONRCHANS, 0)) < 0) {
			perror ("COSAIONRCHANS");
			return -1;
		} else
			printf("%d\n", i);
		return 0;
	} else if (!strcmp(argv[2], "syncppp") && argc == 3) {
		if (do_ioctl(fd, SPPPIOCPPP, 0) == -1) {
			perror("SPPPIOCPPP");
			return -1;
		}
		return 0;
	} else if (!strcmp(argv[2], "hdlc") && argc == 3) {
		if (do_ioctl(fd, SPPPIOCCISCO, 0) == -1) {
			perror("SPPPIOCCISCO");
			return -1;
		}
		return 0;
	} else if (!strcmp(argv[2], "debug") && argc == 3) {
		if (do_ioctl(fd, SPPPIOCDEBUG, 1) == -1) {
			perror("SPPPIOCDEBUG");
			return -1;
		}
		return 0;
	} else if (!strcmp(argv[2], "nodebug") && argc == 3) {
		if (do_ioctl(fd, SPPPIOCDEBUG, 0) == -1) {
			perror("SPPPIOCDEBUG");
			return -1;
		}
		return 0;
	} else if (!strcmp(argv[2], "bmset") && argc == 4) {
		int i = atoi(argv[3]);
		if (do_ioctl(fd, COSAIOBMSET, i) == -1) {
			perror("COSAIOBMSET");
			return -1;
		}
		return 0;
	} else if (!strcmp(argv[2], "bmget") && argc == 3) {
		int i;
		if ((i=do_ioctl(fd, COSAIOBMGET, 0)) == -1) {
			perror("COSAIOBMGET");
			return -1;
		}
		printf("%d\n", i);
		return 0;
	}
	usage(*argv);
	return 0;
}
